import Grid from '@mui/material/Grid'
import { makeStyles } from '@material-ui/core'
import { useState,useEffect } from 'react'
import { useSelector } from 'react-redux'

import HeaderBar from './HeaderBar'
import ChatBar from './ChatBar'
import { getPinnedMessagesMemo } from '../../../redux/pinnedMessages/selector'
import { TRightIsOpen } from '../../../typescript/redux/control/types'
import { unpinAllMessagesById } from '../../../api-data'

const useStyles = makeStyles({
  chatBar: {
    background: 'linear-gradient(to bottom right, #e7f097 , #b1e667,#f4f75e)',
  },
})

interface ICentralBar {
  rightIsOpen: TRightIsOpen,
  chatDivRef: any | null,
  companionId: string
}

const CentralBar = ({rightIsOpen,chatDivRef,companionId}:ICentralBar) => {
  const classes = useStyles()
  const pinnedMessagesMemo = useSelector(getPinnedMessagesMemo)
  const [selectedArr, setSelectedArr] = useState<string[] | []>([])
  const [openPinned, setOpenPinned] = useState<boolean>(false)
  const [isSomeSelected, setIsSomeSelected] = useState<boolean>(false)
  const handleClearSelect = () => {
    selectedArr.length > 0 && setSelectedArr([])
    isSomeSelected && setIsSomeSelected(false)
  }
  const handleUnpinAll = () => {
    openPinned && setOpenPinned(false)
    unpinAllMessagesById(pinnedMessagesMemo.map(({_id}) => _id))
  }
  
  useEffect(() => {
    if(pinnedMessagesMemo.length === 0 && openPinned) setOpenPinned(false)
  }, [openPinned, pinnedMessagesMemo])
  
  useEffect(() => {
    setIsSomeSelected(false)
    setSelectedArr([])
  },[companionId])  

    return ( 
     <Grid item lg={rightIsOpen?8:12}>
        <Grid item lg={12} >
          <HeaderBar chatDivRef={chatDivRef} selectedArr={selectedArr} isSomeSelected={isSomeSelected}
            handleClearSelect={handleClearSelect} setIsSomeSelected={setIsSomeSelected}
            openPinned={openPinned} setOpenPinned={setOpenPinned} pinnedMessagesMemo={pinnedMessagesMemo}/>
        </Grid>          
        <Grid item lg={12} className={classes.chatBar}>
          <ChatBar chatDivRef={chatDivRef} selectedArr={selectedArr} setSelectedArr={setSelectedArr}
            isSomeSelected={isSomeSelected} setIsSomeSelected={setIsSomeSelected}
            openPinned={openPinned} pinnedMessagesMemo={pinnedMessagesMemo} handleUnpinAll={handleUnpinAll}/>
        </Grid>
      </Grid>
    )   
}

export default CentralBar